from time import sleep

from direct.filter.FilterManager import FilterManager
from direct.showbase.ShowBase import ShowBase
from panda3d.core import (
    DirectionalLight,
    Shader,
    loadPrcFileData,
    WindowProperties,
    Texture,
    AmbientLight,
)

WAIT = 0.5

# Init
loadPrcFileData("", "gl-version 4 1")
base = ShowBase()
base.accept("escape", exit)
props = WindowProperties()
display_width = base.pipe.getDisplayWidth()
display_height = base.pipe.getDisplayHeight()
props.setSize(display_width, display_height)
base.win.requestProperties(props)

# Comment or uncomment the following lines to see the glitch effect
manager = FilterManager(base.win, base.cam)
tex = Texture()
quad = manager.renderSceneInto(colortex=tex)
# noinspection PyArgumentList
quad.setShader(
    Shader.load(
        Shader.SL_GLSL,
        vertex="basic.vert",
        fragment=f"quad_glitch.frag",
    )
)  # Set shaders

cube = base.loader.loadModel("cube")
# noinspection PyArgumentList
cube_shader = Shader.load(
    Shader.SL_GLSL,
    vertex="glitch.vert",
    fragment="triangle.frag",
)
cube.setShader(cube_shader)
cube.setShaderInput("u_SpecularBlendStrength", 2.0)
cube.setShaderInput("u_ShadowSpread", 0.01)
cube.setShaderInput("u_MaxShininess", 128)
cube.setShaderInput("u_RimLightIntensity", 1.2)
cube.setShaderInput("u_ShadowSamples", 0)
cube.setShaderInput("u_NumberOfLights", 1)
cube.setTwoSided(False, 1)

cube.setScale(1.0)
cube.setPos(-1, -2, 1)
cube.reparentTo(base.render)

cube_bg = cube.copyTo(base.render)
cube_bg.setScale(6.0)
cube_bg.setPos(0, 5, 0)

# Directional light
directional_light = DirectionalLight("directional_light")
directional_light.setColor((1, 1, 1, 1))  # Set the light color
directional_lens = directional_light.getLens()
directional_lens.setFilmSize(10)  # Set the frustum
directional_lens.setNear(-directional_lens.getFar())  # Adjust the near plane
directional = base.render.attachNewNode(directional_light)
directional.setHpr(-45, -20, 0)  # Set the light direction
directional.node().setShadowCaster(True, 2048, 2048, -10)
base.render.setLight(directional)

# Ambient light
ambient_light = AmbientLight("ambient_light")
ambient_light.setColor((0.1, 0.1, 0.1, 1))
ambient = base.render.attachNewNode(ambient_light)
base.render.setLight(ambient)

base.cam.setPos(0, -10, 0)

base.taskMgr.step()
while True:
    directional.setH(-30)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
    directional.setH(-15)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
    base.taskMgr.step()
    sleep(WAIT)
